/* Default text color for most widgets, but not tooltips (hence the '.' before
 * QLabel */
.QLabel, #configDialog QPushButton, QToolButton, QCheckBox, QComboBox, QSpinBox, QListView {
	color: #ccc;
}

/* Default borders, normal and hover */
QCheckBox::indicator, #configDialog QPushButton, QSlider::groove, QSlider::handle, QListView {
	border: 1px solid #666;
}

QCheckBox::indicator:hover, QToolButton::hover, #configDialog QPushButton:hover, QSlider::groove:hover, QSlider::handle:hover {
	border: 1px solid #ccc;
}

/* Selections */
QComboBox, QLineEdit, QSpinBox, QTextEdit, QListView {
	selection-color: #333;
	selection-background-color: #ccc;
}

/* QToolButton */
QToolButton {
	padding: 2px;
	border: 1px solid transparent;
	border-radius: 4px;
}

QToolButton:pressed {
	background-color:
		qlineargradient(x1:0, y1:0, x2:0, y2:1,
		stop:0 #222, stop: 0.6 black, stop:1 black);
	border: 1px solid #444;
}

/* Text editable widgets */
QComboBox:editable, QSpinBox, QLineEdit, QTextEdit, QListView {
	background-color: #333;
}

/* Containers */
Gwenview--HudWidget {
	background-color:
		qlineargradient(x1:0, y1:0, x2:0, y2:1,
		stop:0 rgba(4, 4, 4, 66%), stop: 0.6 rgba(0, 0, 0, 66%), stop:1 rgba(0, 0, 0, 66%));
	border: 1px solid rgba(100, 100, 100, 66%);
	border-radius: 8px;
}

Gwenview--HudWidget[opaque=true] {
	border-radius: 0;
	background-color:
		qlineargradient(x1:0, y1:0, x2:0, y2:1,
		stop:0 hsv(0, 0, 20%), stop: 0.6 rgb(0, 0, 0), stop:1 rgb(0, 0, 0));
}

#fullScreenBar {
	background-color:
		qlineargradient(x1:0, y1:0, x2:0, y2:1,
		stop:0 #444, stop: 0.6 black, stop:1 black);
	border: 0px;
	border-bottom: 1px solid gray;
	padding: 0px;
}

#fullScreenBar[fullWidth=false] {
	padding-right: 6px;
	border-right: 1px solid gray;
	border-bottom-right-radius: 7px;
}

#configDialog {
	background-color:
		qlineargradient(x1:0, y1:0, x2:0, y2:1,
		stop:0 #444, stop: 0.6 black, stop:1 black);
	border: 1px solid gray;
	border-radius: 0px;
}

/* QCheckBox */
QCheckBox::indicator {
	width: 16px;
	height: 16px;
	border: 1px solid #666;
	padding: 2px;
	border-radius: 3px;
}

QCheckBox::indicator:checked {
	image:url($themeDir/checked.png);
}

/* QListView */
QListView {
	padding: 2px;
}

/* QPushButton */
#configDialog QPushButton {
	background-color:
		qlineargradient(x1:0, y1:0, x2:0, y2:1,
		stop:0 #333, stop: 0.5 black, stop:1 black);
	border-radius: 3px;
	padding: 3px;
}

#configDialog QPushButton:hover {
	background-color:
		qlineargradient(x1:0, y1:0, x2:0, y2:1,
		stop:0 #444, stop: 0.6 black, stop:1 black);
}

#configDialog QPushButton:pressed {
	background-color: #111;
	padding-right: 1px;
	padding-bottom: 1px;
}

/* QSlider */
QSlider::groove {
	background-color:
		qlineargradient(x1:0, y1:0, x2:0, y2:1,
		stop:0 black, stop:1 #444);
	border-radius: 3px;
	height: 4px;
}

QSlider::handle {
	margin-top: -4px;
	margin-bottom: -4px;
	width: 12px;
	background-color:
		qlineargradient(x1:0, y1:0, x2:0, y2:1,
		stop:0 #333, stop: 0.5 black, stop:1 black);
	border-radius: 4px;
}

QSlider::handle:hover {
	background-color:
		qlineargradient(x1:0, y1:0, x2:0, y2:1,
		stop:0 #444, stop: 0.6 black, stop:1 black);
}


/* Misc */
QLabel[isTitle=true] {
	font-weight: bold;
	color: white;
}

#line {
	border: none;
	border-top: 1px solid gray;
	height: 3px;
}

/* Thumbnail Bar */
#thumbnailBarView {
	background-color: black;
	border: 1px solid black;
	border-radius: 2px;
	padding: 0px;
}

#thumbnailBarView::item {
	background-color:
		qlineargradient(x1:0, y1:0, x2:0, y2:1,
		stop:0 #3e3e3e, stop: 1 #101010);
	border-left: 1px solid
		qlineargradient(x1:0, y1:0, x2:0, y2:1,
		stop:0 #4d4d4d, stop: 1 #292929);
	border-right: 1px solid
		qlineargradient(x1:0, y1:0, x2:0, y2:1,
		stop:0 #0d0d0d, stop: 1 #000000);
}

#thumbnailBarView::item:selected {
	background-color: palette(highlight);
	border-left: 1px solid palette(dark);
	border-right: 1px solid palette(dark);
}

#thumbnailBarView QScrollBar:horizontal {
	background-color: transparent;
	height: 10px;
	padding: 0 2px 0px 2px;
	margin: 0 10px 0px 10px;
}

#thumbnailBarView QScrollBar::add-page:horizontal,
#thumbnailBarView QScrollBar::sub-page:horizontal {
	width: 0px;
	border: 0px;
}

#thumbnailBarView QScrollBar::add-line:horizontal {
	width: 10px;
	border: 0px;
	background: transparent;
	subcontrol-position: right;
	subcontrol-origin: margin;
}

#thumbnailBarView QScrollBar::sub-line:horizontal {
	width: 10px;
	border: 0px;
	background: transparent;
	subcontrol-position: left;
	subcontrol-origin: margin;
}

#thumbnailBarView QScrollBar:left-arrow:horizontal {
	background: url($themeDir/scroll-left.png);
	height: 7px;
	width: 7px;
}

#thumbnailBarView QScrollBar:right-arrow:horizontal {
	background: url($themeDir/scroll-right.png);
	height: 7px;
	width: 7px;
}

#thumbnailBarView QScrollBar::handle:horizontal {
	border-radius: 3px;
	min-width: 20px;
	background:
		qlineargradient(x1:0, y1:0, x2:0, y2:1,
		stop: 0 #565656, stop: 0.5 #363636,
		stop: 0.51 #161616, stop: 1 #363636);
	border: 1px solid #363636;
	border-top: 1px solid #6e6e6e;
}

#thumbnailBarView QScrollBar::handle:hover {
	background:
		qlineargradient(x1:0, y1:0, x2:0, y2:1,
		stop: 0 #5e5e5e, stop: 0.5 #3e3e3e,
		stop: 0.51 #1e1e1e, stop: 1 #3e3e3e);
	border: 1px solid #3e3e3e;
	border-top: 1px solid #767676;
}
